// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_Service_hpp
#define anna_comm_Service_hpp

#include <anna/comm/Delivery.hpp>

namespace anna {

namespace comm {

class Server;
class Message;

/**
   Clase que modela un servicio de reparto de carga. Cada servicio de reparto de carga esta asociado
   a un nmero indeterminado de procesos servidores (ver Server) entre los que puede seleccionar a
   la hora de enviar una peticin.

   Para crear un nuevo servicio hay que invocar a Communicator::createService.
*/
class Service : public Delivery {
public:
  /**
     Constructor.
     \param name Nombre lgico del servicio de reparto de carga.
     \param isCritical Indica las caracteristicas del reparto de carga. Debe valor \em true si el
     reparto de carga a crear es crtico o \em false en otro caso.
  */
  Service(const char* name, const bool isCritical) :
    Delivery(name),
    a_isCritical(isCritical) {}

  /**
     Devuelve la caracteristica de criticidad de este reparto.

     @return La caracteristica de criticidad de este reparto.
  */
  bool isCritical() const throw() { return a_isCritical; }

  /**
     Conecta el servidor recibido como parmetro con este reparto.
     \param server Instancia del servidor que vamos a registrar en este reparto.
  */
  void attach(Server* server) throw(RuntimeException);

  /**
     Enva el mensaje recibido como parmetro. El bloque de datos recibido se codifica segun las
     reglas establecidas por el protocolo asociado en el contructor.

     \param message Mensaje que vamos codificar para enviar a la capa de transporte.
     @return Servidor usado en el envio
  */
  comm::Server* send(Message& message) throw(RuntimeException);

  /**
     Enva el mensaje recibido como parmetro. El bloque de datos recibido se codifica segun las
     reglas establecidas por el protocolo asociado en el contructor.

     \param message Mensaje que vamos codificar para enviar a la capa de transporte.
     @return Servidor usado en el envio
  */
  comm::Server* send(Message* message) throw(RuntimeException);

  /**
     Enva el mensaje recibido como parmetro a todos los servidores incluidos en el servicio de reparto de carga
     recibido como parmetro. El bloque de datos recibido se codifica segun las reglas establecidas
     por el protocolo asociado.

     \param message Mensaje que vamos a enviar al todos los procesos que componente el servicio.

     @return Numero de servidores a los que ha enviado el mensaje.
  */
  int broadcast(Message& message) throw();

  /**
     Enva el mensaje recibido como parmetro a todos los servidores incluidos en el servicio de reparto de carga
     recibido como parmetro. El bloque de datos recibido se codifica segun las reglas establecidas
     por el protocolo asociado.

     \param message Mensaje que vamos a enviar al todos los procesos que componente el servicio.

     @return Numero de servidores a los que ha enviado el mensaje.
  */
  int broadcast(Message* message) throw();

  /**
     Devuelve una cadena con la informacin referente a esta maquina.
     @return Una cadena con la informacin referente a esta maquina.
  */
  std::string asString() const throw();

  /**
     Devuelve un nodo XML con la informacin referente a este objeto.
     \param parent Nodo XML a partir del cual introducir la informacin.
     \return Un nodo XML con la informacin referente a este objeto.
  */
  xml::Node* asXML(xml::Node* parent) const throw();

  /**
     Devuelve el recurso remoto apuntado por el iterador recibido como parmetro.
     \param ii Iterador usado para recorrer los recursos asociados a este reparto.
     \return El recurso remoto apuntado por el iterador recibido como parmetro.
  */
  static Server* server(iterator& ii) throw();

  /**
     Devuelve el recurso remoto apuntado por el iterador recibido como parmetro.
     \param ii Iterador usado para recorrer los recursos asociados a este reparto.
     \return El recurso remoto apuntado por el iterador recibido como parmetro.
  */
  static const Server* server(const_iterator& ii) throw();

private:
  const bool a_isCritical;
};

}
}

#endif
