// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_defines_hpp
#define anna_core_util_defines_hpp

// STL
#include <algorithm>
#include <string>
#include <vector>

#include <stdio.h>

#include <stdint.h>
// For cstd0x compatibility we will use stdint.h types instead of std:: ones on cstdint:
//#include <cstdint>  when C++11 available

// For xml representations (xml::Compiler and json::functions::json2xml)
#define ANNA_XML_INDENTATION_SPACES 3


// Decoding helpers
#define DECODE2BYTES_INDX_VALUETYPE(buffer,indx,value_type) ((((value_type)buffer[indx] << 8) & 0xFF00) + ((value_type)buffer[indx+1] & 0x00FF))
#define DECODE3BYTES_INDX_VALUETYPE(buffer,indx,value_type) ((((value_type)buffer[indx] << 16) & 0xFF0000) + (((value_type)buffer[indx+1] << 8) & 0x00FF00) + ((value_type)buffer[indx+2] & 0x0000FF))
#define DECODE4BYTES_INDX_VALUETYPE(buffer,indx,value_type) ((((value_type)buffer[indx] << 24) & 0xFF000000) + (((value_type)buffer[indx+1] << 16) & 0x00FF0000) + (((value_type)buffer[indx+2] << 8) & 0x0000FF00) + ((value_type)buffer[indx+3] & 0x000000FF))

//#define DECODE2BYTES_VALUETYPE(buffer,value_type) DECODE2BYTES_INDX_VALUETYPE(buffer,0,value_type)
//#define DECODE3BYTES_VALUETYPE(buffer,value_type) DECODE3BYTES_INDX_VALUETYPE(buffer,0,value_type)
//#define DECODE4BYTES_VALUETYPE(buffer,value_type) DECODE4BYTES_INDX_VALUETYPE(buffer,0,value_type)


namespace anna {

#ifndef __x86_64__
#undef  __anna64__
#else
#define __anna64__
#endif

//      TYPE                    BITS (bytes)       Format
//      ----------------------- ------------------ ---------------------------
//      unsigned long int       32/64 (4/8)        lu
//      long int                32/64 (4/8)        ld
//
//      unsigned long long int  64 (8)             llu
//      long long int           64 (8)             lld
//
//      float                   32 (4)             f
//      double                  64 (8)             lf
//      long double             80 (10)            Lf
//
//      pointer = S.O. word     32/64              p


/** Alias for unsigned integer with 8 bits */
//typedef uint8_t U8;
typedef unsigned char U8;

/** Alias for signed integer with 8 bits */
//typedef int8_t S8;
typedef char S8;

/** Alias for unsigned integer with 16 bits */
typedef uint16_t U16;

/** Alias for signed integer with 16 bits */
typedef int16_t S16;

/** Alias for unsigned integer with 32 bits */
typedef uint32_t U32;

/** Alias for signed integer with 32 bits */
typedef int32_t S32;

/** Alias for unsigned integer with 64 bits */
typedef uint64_t U64;

/** Alias for signed integer with 64 bits */
typedef int64_t S64;

/** Alias for float: floating number with 32 bits (1-8-23) */
typedef float F32;

/** Alias for double: floating number with 64 bits (1-11-52) */
typedef double F64;

/** Alias for long double: floating number with 80 bits (1-15-64) */
typedef long double F80;

/** Alias for unsigned integer with 32 bits used to contain 24 bits */
typedef U32 U24;


// Communication

/**
* Typedef for socket literal representation: <ip|hostname>:<port>
*/
typedef std::pair <std::string, int> socket_t;

/**
* Typedef for sockets (socket_t) vector
*/
typedef std::vector<socket_t> socket_v;
typedef std::vector<socket_t>::const_iterator socket_v_it;




/**
   Struct for called and calling party number from Q763 Signalling System No. 7  ISDN user part formats and codes

   <pre>

   Called Party Number
              8         7         6         5         4         3         2         1
         |---------|---------|---------|---------|---------|---------|---------|---------|
   1     |   O/E   |              Nature of address indicator                            |
         |---------|---------|---------|---------|---------|---------|---------|---------|
   2     |   INN   |   Numbering plan indicator  |                 spare                 |
         |---------|---------|---------|---------|---------|---------|---------|---------|
   3     |          2nd address signal           |           1st address signal          |
         |---------|---------|---------|---------|---------|---------|---------|---------|
        ...       ...       ...       ...       ...       ...       ...       ...       ...
         |---------|---------|---------|---------|---------|---------|---------|---------|
   m     |         Filler (if necessary)         |           nth address signal          |
         |---------|---------|---------|---------|---------|---------|---------|---------|



   Calling Party Number
              8         7         6         5         4         3         2         1
         |---------|---------|---------|---------|---------|---------|---------|---------|
   1     |   O/E   |              Nature of address indicator                            |
         |---------|---------|---------|---------|---------|---------|---------|---------|
   2     |   NI    |   Numbering plan indicator  |Add.Pres.Restr.Ind |   Screening Ind   |
         |---------|---------|---------|---------|---------|---------|---------|---------|
   3     |          2nd address signal           |           1st address signal          |
         |---------|---------|---------|---------|---------|---------|---------|---------|
        ...       ...       ...       ...       ...       ...       ...       ...       ...
         |---------|---------|---------|---------|---------|---------|---------|---------|
   m     |         Filler (if necessary)         |           nth address signal          |
         |---------|---------|---------|---------|---------|---------|---------|---------|

   </pre>
*/
typedef struct {
  /**  odd/even (1/0) indicator */
  short OddEven;

  /** Return true when have odd number of digits */
  bool isOdd() { return OddEven; }
  /** Return true when have even number of digits */
  bool isEven() { return !isOdd(); }


  /** <pre>
     Nature of address indicator
     0 0 0 0 0 0 0 spare
     0 0 0 0 0 0 1 subscriber number (national use)
     0 0 0 0 0 1 0 unknown (national use)
     0 0 0 0 0 1 1 national (significant) number (national use)
     0 0 0 0 1 0 0 international number

   Called:
     0 0 0 0 1 0 1 network-specific number (national use) ITU-T Q.763 (12/1999) 23
     0 0 0 0 1 1 0 network routing number in national (significant) number format (national use)
     0 0 0 0 1 1 1 network routing number in network-specific number format (national use)
     0 0 0 1 0 0 0 network routing number concatenated with Called Directory Number (national use)
     1 1 0 1 1 1 1 to 0 0 0 1 0 0 1 spare
     1 1 1 1 1 1 0 to 1 1 1 0 0 0 0 reserved for national use
     1 1 1 1 1 1 1 spare

   Calling:
     0 0 0 0 1 1 0 to 1 1 0 1 1 1 1 spare
     1 1 1 0 0 0 0 to 1 1 1 1 1 1 0 reserved for national use
     1 1 1 1 1 1 1 spare
     </pre>
   */
  short NatureOfAddress;

  /** Return true when Nature Of Address is 'subscriber number (national use)' */
  bool NatureOfAddress_SubscriberNumber() { return (NatureOfAddress == 1); }
  /** Return true when Nature Of Address is 'unknown (national use)' */
  bool NatureOfAddress_Unknown() { return (NatureOfAddress == 2); }
  /** Return true when Nature Of Address is 'national (significant) number (national use)' */
  bool NatureOfAddress_NationalNumber() { return (NatureOfAddress == 3); }
  /** Return true when Nature Of Address is 'international number' */
  bool NatureOfAddress_InternationalNumber() { return (NatureOfAddress == 4); }


  /** <pre>
     Internal Network Number indicator (INN) (only for called party number)
     0 routing to internal network number allowed
     1 routing to internal network number not allowed
     </pre>
   */
  short InternalNetworkNumber;

  /** Return true when Internal Network Number Indicator is 'routing to internal network number allowed' */
  bool InternalNetworkNumber_RoutingToInternalNetworkNumberAllowed() { return (InternalNetworkNumber == 0); }
  /** Return true when Internal Network Number Indicator is 'routing to internal network number not allowed' */
  bool InternalNetworkNumber_RoutingToInternalNetworkNumberNotAllowed() { return (InternalNetworkNumber == 1); }


  /** <pre>
     Number Incomplete indicator (NI) (only for calling party number)
     0 complete
     1 incomplete
     </pre>
   */
  short NumberIncomplete;

  /** Return true when Number Incomplete Indicator is 'complete' */
  bool NumberIncomplete_Complete() { return (NumberIncomplete == 0); }
  /** Return true when Number Incomplete Indicator is 'incomplete' */
  bool NumberIncomplete_Incomplete() { return (NumberIncomplete == 1); }


  /** <pre>
     Numbering plan indicator
     0 0 0 spare
     0 0 1 ISDN (Telephony) numbering plan (ITU-T Recommendation E.164)
     0 1 0 spare
     0 1 1 Data numbering plan (ITU-T Recommendation X.121) (national use)
     1 0 0 Telex numbering plan (ITU-T Recommendation F.69) (national use)
     1 0 1 reserved for national use
     1 1 0 reserved for national use
     1 1 1 spare
     </pre>
   */
  short NumberingPlan;

  /** Return true when Numbering Plan is 'ISDN (Telephony) numbering plan (ITU-T Recommendation E.164)' */
  bool NumberingPlan_ISDN() { return (NumberingPlan == 1); }
  /** Return true when Numbering Plan is 'Data numbering plan (ITU-T Recommendation X.121) (national use)' */
  bool NumberingPlan_Data() { return (NumberingPlan == 3); }
  /** Return true when Numbering Plan is 'Telex numbering plan (ITU-T Recommendation F.69) (national use)' */
  bool NumberingPlan_Telex() { return (NumberingPlan == 4); }


  /** <pre>
     Address presentation restricted indicator (only for calling party number)
     0 0 presentation allowed
     0 1 presentation restricted
     1 0 address not available (Note 1) (national use)
     1 1 reserved for restriction by the network
     NOTE 1  If the parameter is included and the address presentation restricted indicator indicates
     address not available, octets 3 to n are omitted, the subfields in items 'OddEven', 'NatureOfAddress',
     'NumberIncomplete' and 'NumberingPlan' are coded with 0's, and the subfield 'Screening' is coded with 11.
     </pre>
   */
  short AddressPresentationRestricted;

  /** Return true when Address Presentation Restricted is 'presentation allowed' */
  bool AddressPresentationRestricted_PresentationAllowed() { return (AddressPresentationRestricted == 0); }
  /** Return true when Address Presentation Restricted is 'presentation restricted' */
  bool AddressPresentationRestricted_PresentationRestricted() { return (AddressPresentationRestricted == 1); }
  /** Return true when Address Presentation Restricted is 'address not available (Note 1) (national use)' */
  bool AddressPresentationRestricted_AddressNotAvailable() { return (AddressPresentationRestricted == 2); }
  /** Return true when Address Presentation Restricted is 'reserved for restriction by the network' */
  bool AddressPresentationRestricted_ReservedForRestrictionByTheNetwork() { return (AddressPresentationRestricted == 3); }


  /** <pre>
     Screening indicator (only for calling party number)
     0 0 reserved (Note 2)
     0 1 user provided, verified and passed
     1 0 reserved (Note 2)
     1 1 network provided
     NOTE 2  Code 00 and 10 are reserved for "user provided, not verified" and "user provided, verified
     and failed" respectively. Codes 00 and 10 are for national use.
     </pre>
   */
  short Screening;

  /** Return true when Screening is 'user provided, verified and passed' */
  bool Screening_UserProvidedVerifiedAndPassed() { return (Screening == 1); }
  /** Return true when Screening is 'network provided' */
  bool Screening_NetworkProvided() { return (Screening == 3); }


  /** <pre>
     BCD digit
     Address signal
     0 0 0 0 digit 0
     0 0 0 1 digit 1
     0 0 1 0 digit 2
     0 0 1 1 digit 3
     0 1 0 0 digit 4
     0 1 0 1 digit 5
     0 1 1 0 digit 6
     0 1 1 1 digit 7
     1 0 0 0 digit 8
     1 0 0 1 digit 9
     1 0 1 0 spare
     1 0 1 1 code 11
     1 1 0 0 code 12
     1 1 0 1 spare
     1 1 1 0 spare
     1 1 1 1 ST (for called party number, spare in calling party number)
     The most significant address signal is sent first. Subsequent address signals are sent in
     successive 4-bit fields.

     Filler: In case of an odd number of address signals, the filler code 0000 is inserted after the last
             address signal.
     </pre>
   */
  std::string Digits;

  void reset() {
    OddEven = 0;
    NatureOfAddress = 0;
    InternalNetworkNumber = 0;
    NumberIncomplete = 0;
    NumberingPlan = 0;
    AddressPresentationRestricted = 0;
    Screening = 0;
    Digits = "";
  }

  /**
  * Class string representation
  *
  * @param calledOrCalling Boolean about being called party number or calling one
  *
  * @return String with class content
  */
  std::string asString(bool calledOrCalling) {
    std::string result;
    char aux [16];
    result = "OddEven: ";
    sprintf(aux, "%d", OddEven); result += std::string(aux);
    result += " | NatureOfAddress: ";

    if(calledOrCalling) {
      sprintf(aux, "%d", NatureOfAddress); result += std::string(aux);
      result += " | InternalNetworkNumber: ";
    } else {
      sprintf(aux, "%d", InternalNetworkNumber); result += std::string(aux);
      result += " | NumberIncomplete: ";
    }

    sprintf(aux, "%d", NumberIncomplete); result += std::string(aux);
    result += " | NumberingPlan: ";
    sprintf(aux, "%d", NumberingPlan); result += std::string(aux);

    if(!calledOrCalling) {
      result += " | AddressPresentationRestricted: ";
      sprintf(aux, "%d", AddressPresentationRestricted); result += std::string(aux);
      result += " | Screening: ";
      sprintf(aux, "%d", Screening); result += std::string(aux);
    }

    result += " | Digits: ";
    result += (Digits != "") ? Digits : "<null>";
    return result;
  }

//   int getEncodedLength() {
//
//      bool filler = OddEven;
//      bool hasDigits = (Digits.size() > 0);
//      return (2 + Digits.size()/2 + ((hasDigits && filler) ? 1:0));
//   }

} isup_number_t; // ISDN user part parameters


/**
* IANA Address Family Numbers
* @see http://www.iana.org/assignments/address-family-numbers/address-family-numbers.xml
*/
typedef struct {
  enum _v {
    //Number    Description                                          Reference
    //------    ---------------------------------------------------- ---------
    //     0    Reserved
    IPv4 = 1, //IP (IP version 4)
    IPv6 = 2, //IP6 (IP version 6)
    //     3    NSAP
    //     4    HDLC (8-bit multidrop)
    //     5    BBN 1822
    //     6    802 (includes all 802 media plus Ethernet "canonical format")
    //     7    E.163
    E164 = 8  //E.164 (SMDS, Frame Relay, ATM)
    //     9    F.69 (Telex)
    //    10    X.121 (X.25, Frame Relay)
    //    11    IPX
    //    12    Appletalk
    //    13    Decnet IV
    //    14    Banyan Vines
    //    15    E.164 with NSAP format subaddress           [UNI-3.1] [Andy_Malis]
    //    16    DNS (Domain Name System)
    //    17    Distinguished Name                                    [Charles_Lynn]
    //    18    AS Number                                             [Charles_Lynn]
    //    19    XTP over IP version 4                                 [Mike_Saul]
    //    20    XTP over IP version 6                                 [Mike_Saul]
    //    21    XTP native mode XTP                                   [Mike_Saul]
    //    22    Fibre Channel World-Wide Port Name                   [Mark_Bakke]
    //    23    Fibre Channel World-Wide Node Name                   [Mark_Bakke]
    //    24    GWID                                                 [Subra_Hegde]
    //    25    AFI for L2VPN information [RFC4761][RFC6074]
    //    26-16383 Unassigned
    //    16384 EIGRP Common Service Family [Donnie_Savage] 2008-05-13
    //    16385 EIGRP IPv4 Service Family [Donnie_Savage] 2008-05-13
    //    16386 EIGRP IPv6 Service Family [Donnie_Savage] 2008-05-13
    //    16387 LISP Canonical Address Format (LCAF) [David_Meyer] 2009-11-12
    //    16388-32767 Unassigned
    //    32768-65534 Unassigned
    //    65535 Reserved
  };

  /**
  * Version description
  * @param v Version type
  * @return Version description
  */
  static const char* asText(const _v v) throw() { // anna_declare_enum is not safe, because labels don't have to match a sequence
    if(v == IPv4) return "IPv4";

    if(v == IPv6) return "IPv6";

    if(v == E164) return "E164";

    return NULL;
  }

} iana_address_version_t;


/**
   Struct for IANA Addresses
*/
typedef struct {

  /** address version */
  U16 Version;

  /** address printable value. No checkings are done regarding specific version (application responsability) */
  std::string Value;


  /** Gets the address version */
  const U16 & getVersion() const throw() { return Version; }

  /** Gets the address printable value */
  const char * getValue() const throw() { return Value.c_str(); }


  // Helpers

  /** Return true when is an IPv4 address */
  bool isIPv4() const throw() { return ((iana_address_version_t::_v)Version == iana_address_version_t::IPv4); }
  /** Return true when is an IPv6 address */
  bool isIPv6() const throw() { return ((iana_address_version_t::_v)Version == iana_address_version_t::IPv6); }
  /** Return true when is an E164 (SMDS, Frame Relay, ATM) address */
  bool isE164() const throw() { return ((iana_address_version_t::_v)Version == iana_address_version_t::E164); }

  /** Sets version for IPv4 address and address itself. Checking is not performed (could assign IPv6 instead ...) */
  void setIPv4(const char *value) throw() { Version = iana_address_version_t::IPv4; Value = value ? value : ""; }

  /** Sets version for IPv6 address and address itself. Checking is not performed (could assign IPv4 instead ...) */
  void setIPv6(const char *value) throw() { Version = iana_address_version_t::IPv6; Value = value ? value : ""; }

  /** Sets version for E164 address and address itself. Checking is not performed ... */
  void setE164(const char *value) throw() { Version = iana_address_version_t::E164; Value = value ? value : ""; }


  /**
  * Class string representation
  *
  * @return String with class content
  */
  std::string asString() const throw() {
    std::string result;
    result += Value.c_str(); // assume that all IANA addresses have a printable representation
    result += " (";
    const char *versionAsText = iana_address_version_t::asText((iana_address_version_t::_v)Version);

    if(versionAsText) {
      result += versionAsText;
      result += ", ";
    }

    result += "IANA version '";
    char aux [16];
    sprintf(aux, "%d", Version);
    result += aux;
    result += "')";
    return result;
  }

} iana_address_t;


};


#endif
