// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_comm_OamModule_hpp
#define anna_diameter_comm_OamModule_hpp


// Project
#include <anna/core/Singleton.hpp>
#include <anna/core/define.autoenum.hpp>
#include <anna/core/oam/Module.hpp>

// Standard
#include <string>


namespace anna {

namespace diameter {

namespace comm {


class OamModule : public anna::oam::Module, public anna::Singleton <OamModule> {

public:

  struct Alarm {

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Transferable Alarms:    preffix = c_                                                                     //
    // Grouped Alarms:         preffix = g_                                                                     //
    //                         Be careful: this alarms take more than one place (better define them at the end) //
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    enum _v {
      // Enumerated labels represent english text and formats. Final order is not restricted to this in any language, but, is useful to
      //  remember the names at CSL. For example, we should have CSL="primary_address,secondary_address", and then the database text
      //  could have any composition: "Fail to deliver on secondary address '${secondary_address:%s}' after failing over primary '${primary_address:%s}'"

      None = -1,

      /* Connectivity */
      // clients
      UnableToDeliverDiameterMessageToEntityDefinedAs__s__, // 'address1:port1 address2:port2 ...'
      UnableToDeliverDiameterMessageToEntityDefinedAsPrimary__s__AndSecondary__s__, // 'Primary Server address1:port1 and Secondary Server address2:port2'
      RequestSentOnClientSessionExpired,
      RequestSentOnServerSessionExpired,
      AnswerReceivedOnClientSessionUnknown,
      AnswerReceivedOnServerSessionUnknown,
      c_LostAvailabilityOverClientSessionWithServer__s__,
      c_LostAvailabilityOverClientSessionWithServer__s__ClientSessionId__d__,
      c_LostAvailabilityOverServerDefinedAs__s__, // 'address:port'
      c_LostAvailabilityOverEntityDefinedAs__s__, // 'address1:port1 address2:port2 ...'
      c_LostAvailabilityOverEntityDefinedAsPrimary__s__AndSecondary__s__, // 'Primary Server address1:port1 and Secondary Server address2:port2'
      c_LostAvailabilityOverEntitiesForEngineWithClassName__s__,
      // servers
      UnableToDeliverDiameterMessageToClientFromLocalServer__s__, // 'address:port'
      LostConnectionForServerSessionAtLocalServer__s__,
      LostConnectionForServerSessionAtLocalServer__s__ServerSessionId__d__,
      UnbindConnectionForServerSessionAtLocalServer__s__DueToInactivityTimeAnomaly,
      UnbindConnectionForServerSessionAtLocalServer__s__ServerSessionId__d__DueToInactivityTimeAnomaly,
      c_LostAvailabilityOverLocalServerDefinedAs__s__, // 'address:port'
      c_LostAvailabilityOverLocalServersForEngineWithClassName__s__
    };

    anna_declare_enum(Alarm);
  };

  struct Counter {
    enum _v
    // In order to simplify, the literals assigned to
    //  enum-type counters are equivalent to spanich specification ones
    {
      None = -1,

      /* Main */
      // receptions
      RequestReceived,
      AnswerReceived,
      RequestReceivedOnClientSession,
      AnswerReceivedOnClientSession,
      RequestReceivedOnServerSession,
      AnswerReceivedOnServerSession,
      // sendings
      RequestSentOK,
      RequestSentNOK,
      AnswerSentOK,
      AnswerSentNOK,
      RequestSentOnClientSessionOK,
      RequestSentOnClientSessionNOK,
      AnswerSentOnClientSessionOK,
      AnswerSentOnClientSessionNOK,
      RequestSentOnServerSessionOK,
      RequestSentOnServerSessionNOK,
      AnswerSentOnServerSessionOK,
      AnswerSentOnServerSessionNOK,

      // expirations
      RequestSentExpired,
      RequestSentOnClientSessionExpired,
      RequestSentOnServerSessionExpired,

      // retransmissions
      RequestRetransmitted,
      RequestRetransmittedOnClientSession,
      RequestRetransmittedOnServerSession,

      // unknown received answers
      AnswerReceivedUnknown,
      AnswerReceivedOnClientSessionUnknown,
      AnswerReceivedOnServerSessionUnknown,

      /* Capabilities Exchange */
      // clients
      CERSentOK,
      CERSentNOK,
      CEAReceived,
      // servers
      CERReceived,
      CEASentOK,
      CEASentNOK,

      /* Diameter Keepalive */
      // clients
      DWRSentOK,
      DWRSentNOK,
      DWAReceived,
      // servers
      DWRReceived,
      DWASentOK,
      DWASentNOK,

      /* Peer disconnection */
      // clients
      DPRSentOK,
      DPRSentNOK,
      DPAReceived,
      // servers
      DPRReceived,
      DPASentOK,
      DPASentNOK,

      /* server socket operations (enable/disable listening port for any local server) */
      ServerSocketsOpened,
      ServerSocketsClosed,

      /* Connectivity */
      // clients
      UnableToDeliverOverEntity,
      LostAvailabilityOverClientSession,
      RecoveredAvailabilityOverClientSession,
      LostAvailabilityOverServer,
      RecoveredAvailabilityOverServer,
      LostAvailabilityOverEntity,
      RecoveredAvailabilityOverEntity,
      LostAvailabilityOverEngineForEntities,
      RecoveredAvailabilityOverEngineForEntities,
      // servers
      UnableToDeliverToClient,
      LostConnectionForServerSession,
      UnbindConnectionForServerSessionDueToInactivityTimeAnomaly,
      CreatedConnectionForServerSession,
      LostAvailabilityOverLocalServer,
      RecoveredAvailabilityOverLocalServer,
      LostAvailabilityOverEngineForLocalServers,
      RecoveredAvailabilityOverEngineForLocalServers
    };

    anna_declare_enum(Counter);
  };

  /* virtual */std::string getDefaultInternalAlarmDescription(const int & alarmType) const throw() { return Alarm::asCString((Alarm::_v)alarmType); }
  /* virtual */std::string getDefaultInternalCounterDescription(const int & counterType) const throw() { return Counter::asCString((Counter::_v)counterType); }


private:

  // private constructor
  OamModule() : anna::oam::Module("diameter::comm oam module") {};


  friend class anna::Singleton <OamModule>;
};

}
}
}

#endif

