// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_comm_ReceiverFactoryImpl_hpp
#define anna_diameter_comm_ReceiverFactoryImpl_hpp

#include <anna/comm/Receiver.hpp>
#include <anna/comm/ReceiverFactory.hpp>
#include <anna/core/util/Recycler.hpp>


namespace anna {

namespace diameter {

namespace comm {

class R;

/**
   Default implementation for receivers factory specific for diameter

   \param R Reference data member (client session or server session)
   \param T Receiver class instantiated by this factory. Must define empty constructor, #setReference setter and
   a #className method returning class name as:

   \code
      static const char* className () throw ();
   \encode
*/
template <typename R, typename T> class ReceiverFactoryImpl : public anna::comm::ReceiverFactory {
public:
  /**
     Constructor.
  */
  ReceiverFactoryImpl(R *reference) : anna::comm::ReceiverFactory(T::className()), a_reference(reference) {;}

private:
  R *a_reference;
  anna::Recycler<T> a_receivers;

  anna::comm::Receiver* do_create() throw() {
    T *result = a_receivers.create();
    result->setReference(a_reference);
    return result;
  }

  void do_release(anna::comm::Receiver* receiver) throw() { a_receivers.release(static_cast <T*>(receiver)); }
};

}
}
}

#endif

