// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_Request_hpp
#define anna_http_Request_hpp

#include <anna/http/Message.hpp>
#include <anna/http/Method.hpp>

namespace anna {

namespace http {

/**
   Clase que modela las peticiones HTTP segun la RFC 2616.
*/
class Request : public Message {
public:
  /**
     Constructor.
  */
  explicit Request() : Message(Message::Type::Request) {;}

  /**
     Devuelve el metodo asociado a esta peticion.
     \return el metodo asociado a esta peticion.
  */
  Method::Type::_v getMethod() const throw() { return a_method; }

  /**
     Devuelve la URI asociado a esta peticion.
     \return la URI asociado a esta peticion.
  */
  const std::string& getURI() const throw() { return a_uri; }

  /**
     Establece el metodo de esta peticion.
     \param method Metodo a establecer para la peticion.
  */
  void setMethod(const Method::Type::_v method) throw() { a_method = method; }

  /**
     Establece el URI de esta peticion.
     \param uri URI a establecer para la peticion.
  */
  void setURI(const std::string& uri) throw() { a_uri = uri; }

  /**
     Devuelve una cadena con toda la informacion relevante de este objeto.
     \return una cadena con toda la informacion relevante de este objeto.
  */
  std::string asString() const throw();

private:
  Method::Type::_v a_method;
  std::string a_uri;

  std::string codeStartLine() const throw(anna::RuntimeException);
};

}
}

#endif

