// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_timex_TimeEventObserver_hpp
#define anna_timex_TimeEventObserver_hpp

namespace anna {

namespace timex {

class TimeEvent;
class Engine;

/**
   Clase base que se encarga de recibir las notificaciones basicas referentes al timex::TimeEvent.

   Hemos comprobado que casi todos los procesos hay alguna entidad que se encarga de crear
   los timex::TimeEvent que van siendo necesarios, normalmente usando un anna::Recycler<T>,
   por lo que como minimo esta entidad tiene que preocuparse por cuando expiran los eventos
   temporales y por cuando se caducan, para poder liberar su espacio.

   Para facilitar el tratamiento de notificaciones vamos a incorporar esta clase, por ahora solo
   recibe notificaciones de cuando se deja de usar un timex::TimeEvent, pero en el futuro podria
   recibir mas eventos.

   \see timex::TimeEvent::setObserver
*/
class TimeEventObserver {
public:
  /**
     Constructor
     @param name Nombre logico de este alojador.
  */
  TimeEventObserver(const char* name) : a_name(name) {}

  /**
     Devuelve una cadena con la informacion referente a este temporizador.
     \return Una cadena con la informacion referenta a este temporizador.
  */
  std::string asString() const
  throw() {
    std::string msg("timex::TimeEventObserver { Name: ");
    msg += a_name;
    return msg += " }";
  }

protected:
  /**
     Metodo virtual que debe reimplmentar para liberar el espacio reservado para el
     evento temporal recibido como parametro.

     Se invoca cuando el timex::Engine deja de usar el evento temporal.

     \param timeEvent Evento temporal a liberar.
  */
  virtual void release(timex::TimeEvent* timeEvent) throw() = 0;

private:
  std::string a_name;

  friend class Engine;
};

}
}

#endif


