// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter.comm/Transport.hpp>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/DataBlock.hpp>
#include <anna/comm/Message.hpp>




using namespace std;
using namespace anna::diameter::comm;

anna::comm::TransportFactoryImpl <Transport> Transport::st_factory;

Transport::Transport() : anna::comm::Transport() {
  setInputMessage(new anna::comm::Message());
}

Transport::~Transport() {
  delete getInputMessage();
}

/*
 * La longitud se calcula con los primeros 4 bytes (el primer byte se ignora, pues es la version diameter)
 */
int Transport::calculeSize(const anna::DataBlock& dataBlock)
throw(anna::RuntimeException) {
  const int size = dataBlock.getSize();

  if(size < 4)
    return -1;

  const char* data = dataBlock.getData();
  int result = (((unsigned int)data[1] << 16) & 0xFF0000) +
               (((unsigned int)data[2] << 8)  & 0x00FF00) +
               (((unsigned int)data[3])    & 0x0000FF);
  return result;
}

const anna::DataBlock& Transport::code(anna::comm::Message& message)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod ttmm(anna::Logger::Local7, "anna::diameter::comm::Transport", "code", ANNA_FILE_LOCATION));
  a_forCode = message.code();
  return a_forCode;
}

//--------------------------------------------------------------------------------
// Los 4 primeros bytes contienen la longitud del mensaje.
// Cada ClientSocket tiene su anna::diameter::comm::Transport asociado
//--------------------------------------------------------------------------------
const anna::comm::Message* Transport::decode(const anna::DataBlock& input)
throw(anna::RuntimeException) {
  anna::comm::Message* inputMessage = static_cast <anna::comm::Message*>(getInputMessage());
  inputMessage->setBody(input);
  return inputMessage;
}
