// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/http/parser/ReadHeader.hpp>

#include <anna/http/Transport.hpp>
#include <anna/http/functions.hpp>
#include <anna/http/internal/defines.hpp>
#include <anna/http/Header.hpp>
#include <anna/http/internal/EncodedBlock.hpp>

using namespace std;
using namespace anna;

//----------------------------------------------------------------------------------------------------
// Si el mensaje HTTP no contiene la etiqueta content-length => el mensaje se acaba cuando llegue
// el primer crlf en solitario, es decir, la primera linea vacia.
//
// (1) El valor que acompania al Header es opcional ... solo verificamos que venga en el caso de
// que sea el ContentLength.
// (2) Hay una probabilidad muy alta de que la fecha contenga algun ':' asi que hay que tratarlo
//     de forma distinta al resto de los header.
//
// Recordar que el ContentLength indica la longitud del cuerpo del mensaje (si existe) =>
// la longitud total del mensaje sea <la longitud de la cabecera> + <longitud del body>.
//
// (3) version 1.0.5 - El servidor de aplicaciones tomcat envia mensajes HTTP sin el Header Content-Length, con lo que
// aplicando la regla de la RFC siempre obtendremos un cuerpo de mensaje vacio.
// Segun la RFC cuando no llega este indicador el mensaje termina con los primeros 0xd0xa consecutivos.
// Para permitir la interconexion de nuestros sistemas con este servidor de aplicaciones, vamos a modificar el comportamiento
// del parser y considerando el fin del mensaje el fin del paquete recibido como parametro.
//
// (4) version 1.0.7 - Los header no reconocidos los guarda como extensiones.
//----------------------------------------------------------------------------------------------------
/*virtual*/
int http::parser::ReadHeader::processLine(http::Transport& transport, const DataBlock& dataBlock, const http::Token& line) const
throw(RuntimeException) {
  if(line.getSize() == 0) {                                                  // (3)
    const int size = line.calculeOffset(dataBlock) + sizeEndOfLine;
    setState(transport, ClassType::WaitMessage);
    return size;
  }

  const Tokenizer& tokenizer = transport.split(line, ':');

  Tokenizer::const_iterator ii = tokenizer.begin();

  const Token* token = tokenizer [ii];

  const Header::Type::_v typeHeader = Header::asType(token);

  Header* header = NULL;

  if(typeHeader == Header::Type::None) {
    if(token == NULL) {
      string msg("http::parser::ReadHeader::processLine | Unreconized header identifier: ");
      msg += line.getStringValue();
      throw RuntimeException(msg, ANNA_FILE_LOCATION);
    }

    header = transport.getInputMessage()->createHeader(token->getStringValue());
  } else if(typeHeader != Header::Type::TransferEncoding)
    header = transport.getInputMessage()->createHeader(typeHeader);

  token = tokenizer [++ ii];                                                            // (1);

  if(typeHeader == Header::Type::ContentLength) {
    if(token == NULL)
      throw RuntimeException("http::parser::ReadHeader::processLine | Missing body length at Header ContentLength", ANNA_FILE_LOCATION);

    transport.setContentLength(token->getIntegerValue());
    header->setValue(token);
    setState(transport, ClassType::WaitEndOfHeader);
  } else if(typeHeader == Header::Type::TransferEncoding) {
    EncodedBlock* encodedBlock = transport.getEncodedBlock();

    if(token == NULL)
      throw RuntimeException("http::parser::ReadHeader::processLine | Missing encode mode at Transfer-Encoding Header", ANNA_FILE_LOCATION);

    if(token->match("chunked")) {
      encodedBlock->setType(EncodedBlock::Type::Chunked);
      setState(transport, ClassType::WaitChunkSize);
    }
  } else if(token != NULL)  {                         // (2)
    const int offset = token->calculeOffset(line);
    const int size = line.getSize() - offset;
    Token date;
    date.setValue(line.getData() + offset, size);
    header->setValue(&date);
  } else
    header->setValue(token);                                          // (4)

  if(header != NULL && Logger::isActive(Logger::Debug)) {
    Logger::debug(header->asString(), ANNA_FILE_LOCATION);
  }

  return -1;
}

