// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/config/defines.hpp>

#include <libxml/parser.h>

#include <anna/xml/Document.hpp>
#include <anna/xml/Parser.hpp>

using namespace std;
using namespace anna;
using namespace anna::xml;

Document::Document() :
  anna::DataBlock(true),
  a_handle(NULL),
  a_encoding(NULL),
  a_version(NULL),
  a_contentIsCString(false),
  a_asCString(NULL),
  a_parser(NULL)
{;}

Document::~Document() {
  if(a_handle != NULL) {
    xmlFreeDoc(a_handle);
    a_handle = NULL;
  }

  delete a_parser;
  delete a_encoding;
  delete a_version;
}

void Document::initialize(const char* content)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tf("anna::xml::Document", "initialize (char*)", ANNA_FILE_LOCATION));
  clear();
  a_handle = do_initialize(content);
  setEncoding((char*) a_handle->encoding);
  setVersion((char*) a_handle->version);
}

void Document::initialize(const DataBlock& content)
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tf("anna::xml::Document", "initialize (DataBlock)", ANNA_FILE_LOCATION));
  clear();
  a_handle = do_initialize(content);
  setEncoding((char*) a_handle->encoding);
  setVersion((char*) a_handle->version);
}

const xml::Node* Document::parse()
throw(RuntimeException) {
  if(a_parser == NULL)
    a_parser = new Parser;

  return a_parser->apply(*this);
}

const xml::Node* Document::parse(const xml::DTD& dtd)
throw(RuntimeException) {
  if(a_parser == NULL)
    a_parser = new Parser;

  return a_parser->apply(*this, dtd);
}

void Document::clear()
throw() {
  if(a_handle != NULL) {
    xmlFreeDoc(a_handle);
    a_handle = NULL;
  }

  if(a_encoding != NULL)
    a_encoding->clear();

  if(a_version != NULL)
    a_version->clear();

  DataBlock::clear();
  a_contentIsCString = false;
}

const char* Document::getEncoding() const
throw() {
  return (a_encoding == NULL) ? NULL : ((a_encoding->empty() == true) ? NULL : a_encoding->c_str());
}

const char* Document::getVersion() const
throw() {
  return (a_version == NULL) ? NULL : ((a_version->empty() == true) ? NULL : a_version->c_str());
}

const char* Document::getContentAsCString() const
throw(RuntimeException) {
  const char* result = NULL;

  if(a_contentIsCString == false) {
    Document* _this = const_cast <Document*>(this);

    if(a_asCString == NULL)
      _this->a_asCString = new DataBlock(true);

    *(_this->a_asCString) = *this;
    *(_this->a_asCString) += '\0';
    result = a_asCString->getData();
  } else
    result = getData();

  return result;
}

void Document::setEncoding(const char* encoding)
throw() {
  if(a_encoding == NULL) {
    if(encoding != NULL)
      a_encoding = new string(encoding);
  } else {
    if(encoding != NULL)
      *a_encoding = encoding;
    else
      a_encoding->clear();
  }
}

void Document::setVersion(const char* version)
throw() {
  if(a_version == NULL) {
    if(version != NULL)
      a_version = new string(version);
  } else {
    if(version != NULL)
      *a_version = version;
    else
      a_version->clear();
  }
}
