// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <string>
#include <anna/time/Date.hpp>
#include <anna/time/functions.hpp>

#include <gtest/gtest.h>

using namespace std;
using namespace anna;


TEST(date, Conversions)
{
  anna::time::functions::initialize();
  anna::time::Date current, current2;
  current.setNow();
  unsigned int ntptime = current.getNtpTimestamp();
  time_t unixtime = current.getUnixTimestamp();
  current2.storeNtp(ntptime);
  std::string ntpStr = current2.asString();
  current2.storeUnix(unixtime);
  std::string unixStr = current2.asString();
  EXPECT_EQ(ntpStr, unixStr);

  anna::time::Date myBirth("CET"); // 19 December 1974, 10:00 (GMT+1 = UTC + 1 = CET)
  myBirth.store("19741219101500"); // yyyymmddHHmmss
  anna::time::Date same_moment_canary_island("GMT");
  same_moment_canary_island.store(myBirth.getUnixTimestamp());
  myBirth.setTz("GMT");
  EXPECT_EQ(same_moment_canary_island.yyyymmddHHmmss(), myBirth.yyyymmddHHmmss());

  anna::time::Date birthday("EET");
  birthday.store(same_moment_canary_island.getTm(), "GMT");  // TZ origin = "GMT"
  EXPECT_EQ(birthday.getUnixTimestamp(), myBirth.getUnixTimestamp());

  myBirth.setTz();
  birthday.setTz();
  EXPECT_EQ(myBirth.asString(), birthday.asString());
  EXPECT_EQ(myBirth.yyyymmddHHmmss(), birthday.yyyymmddHHmmss());

  myBirth.setTz("GMT");
  EXPECT_EQ(myBirth.yyyymmddHHmmss(), "19741219091500");

  // Adding 18 years to 'myBirth':
  same_moment_canary_island.setTz("CET");
  anna::time::Date eighteen("CET");
  struct tm Tm = myBirth.getTm();
  Tm.tm_year += 18;
  eighteen.store(Tm, "GMT");
  EXPECT_TRUE(eighteen >= same_moment_canary_island);
  EXPECT_EQ(eighteen.yyyymmddHHmmss(), "19921219101500");
}



int main(int argc, char **argv)
{
    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}

